<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:ms="urn:schemas-microsoft-com:xslt">
  <xsl:include href="_common.xsl"/>
  <xsl:include href="_dependencies.xsl"/>

  <!-- Type of object for header -->
  <xsl:variable name="objectType" select="'Object'"/>
  
  <!-- Path for object -->
  <xsl:variable name="objectPath" select="'Objects'"/>

  <!-- Root -->
  <xsl:template match="/DataDocumenter">
    <xsl:variable name="pathPart" select="Breadcrumbs/Breadcrumb[last()]/@src"/>
      <div data-page="{$pathPart}{$ext}">
        <xsl:apply-templates select="Breadcrumbs"/>
        <xsl:apply-templates select="Table | View"/>

        <xsl:call-template name="applyDependenciesEvents"/>
    </div>
  </xsl:template>

  <!-- Object information for table or view -->
  <xsl:template match="Table | View">
    <xsl:call-template name="objectInformation">
      <xsl:with-param name="objectType" select="$objectType"/>
    </xsl:call-template>

    <xsl:apply-templates select="Columns"/>
    <xsl:apply-templates select="Relations"/>
    <xsl:apply-templates select="UniqueKeys"/>
    <xsl:apply-templates select="Triggers"/>
    <xsl:apply-templates select="Dependencies"/>

    <xsl:call-template name="metadata"/>
  </xsl:template>

  <!-- Columns -->
  <xsl:template match="Columns">
    <xsl:choose>
      <xsl:when test="./*">
        <table class="table sortable">
          <caption>Columns</caption>
          <thead>
            <tr>
              <th style="width: 40px !important"></th>
              <xsl:choose>
                <xsl:when test="$objectType = 'Table'">
                  <th style="width: 40px !important">PK</th>
                </xsl:when>
              </xsl:choose>
              <th style="width: 30%">Name</th>
              <th style="width: 170px">Data type</th>
              <th style="width: 15px !important">Null</th>
              <th class="only-desktop">Attributes</th>
              <th class="only-desktop">Description</th>
            </tr>
          </thead>
          <tbody>
            <xsl:apply-templates select="Column"/>
          </tbody>
        </table>
      </xsl:when>
    </xsl:choose>
  </xsl:template>

  <!-- Column -->
  <xsl:template match="Column">
    <tr>
      <td>
        <xsl:value-of select="OrdinalPosition"/>
      </td>
      <xsl:choose>
        <xsl:when test="$objectType = 'Table'">
          <xsl:choose>
            <xsl:when test="PK = 'PK'">
              <td>
                <i class="icon-key no-margin" title="Primary key"></i>
              </td>
            </xsl:when>
            <xsl:otherwise>
              <td></td>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
      </xsl:choose>

      <td>
        <xsl:value-of select="Name"/>
        <xsl:if test="string-length(Title)>0">
          (<xsl:value-of select="Title"/>)
        </xsl:if>
      </td>
      <td>
        <xsl:value-of select="DataType"/>
      </td>
      <td>
        <xsl:if test="IsNullable = 'true'">
            <i class="icon-check no-margin" title="Nullable"></i>
        </xsl:if>
      </td>
      <td class="only-desktop">
        <xsl:call-template name="columnAttributes"/>
      </td>
      <td class="only-desktop">
        <xsl:call-template name="insertHtmlNewLines">
          <xsl:with-param name="value" select="Description"/>
        </xsl:call-template>
      </td>
    </tr>

    <xsl:if test="string-length(Description)>0
      or IsIdentity = 'true'
      or string-length(DefaultValue)>0
      or (IsComputed = 'true' and string-length(ComputedFormula)>0)">
      <tr class="only-mobile">
        <td colspan="5">
          <xsl:call-template name="insertHtmlNewLines">
            <xsl:with-param name="value" select="Description"/>
          </xsl:call-template>

          <xsl:if test="string-length(Description)>0
              and (IsIdentity = 'true'
                or string-length(DefaultValue)>0
                or (IsComputed = 'true' and string-length(ComputedFormula)>0))">
            <br></br>
          </xsl:if>

          <xsl:call-template name="columnAttributes"/>
        </td>
      </tr>
    </xsl:if>
  </xsl:template>

  <!-- Column Attributes -->
  <xsl:template name="columnAttributes">
    <xsl:if test="IsIdentity = 'true'">
      Identity / Auto increment column
      
      <xsl:if test="string-length(DefaultValue)>0 or (IsComputed = 'true' and string-length(ComputedFormula)>0)">
        <br/>
      </xsl:if>
    </xsl:if>
    <xsl:if test="string-length(DefaultValue)>0">
      Default:
      <xsl:call-template name="insertHtmlNewLines">
        <xsl:with-param name="value" select="DefaultValue"/>
      </xsl:call-template>
      
      <xsl:if test="IsComputed = 'true' and string-length(ComputedFormula)>0">
        <br/>
      </xsl:if>
    </xsl:if>
    <xsl:if test="IsComputed = 'true' and string-length(ComputedFormula)>0">
      Computed:
      <xsl:call-template name="insertHtmlNewLines">
        <xsl:with-param name="value" select="ComputedFormula"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <!-- Relations -->
  <xsl:template match="Relations">
    <xsl:choose>
      <xsl:when test="./* and (RelationsTo/* or RelationsFrom/*)">
        <table class="table sortablee">
          <caption>Relations</caption>
          <thead>
            <tr>
              <th>Foreign table</th>
              <th style="width: 40px !important"></th>
              <th>Primary table</th>
              <th class="only-desktop">Join</th>
              <th class="only-desktop">Title / Name / Description</th>
            </tr>
          </thead>
          <tbody>
            <xsl:apply-templates select="RelationsFrom/Relation"/>
            <xsl:apply-templates select="RelationsTo/Relation"/>
          </tbody>
        </table>
      </xsl:when>
    </xsl:choose>
  </xsl:template>

  <!-- Relation to -->
  <xsl:template match="RelationsTo/Relation">
    <tr>
      <td>
        <xsl:call-template name="relationLink"/>
      </td>
      
      <!-- relation icon -->
      <td>
        <xsl:choose>
          <xsl:when test="Type = 'USER'">
            <i class="icon-user-relation no-margin" title="User defined relation"></i>
          </xsl:when>
          <xsl:otherwise>
            <i class="icon-relation no-margin" title="Relation"></i>
          </xsl:otherwise>
        </xsl:choose>
      </td>

      <td>
        <xsl:choose>
          <xsl:when test="$objectType = 'Table'">
            <xsl:value-of select="/DataDocumenter/Table/ObjectName"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="/DataDocumenter/View/ObjectName"/>
          </xsl:otherwise>
        </xsl:choose>
      </td>
      <td class="only-desktop">
        <xsl:apply-templates select="Join/Condition"/>
      </td>
      <td class="only-desktop">
        <xsl:if test="string-length(Title)>0">
          <xsl:value-of select="Title"/>
          <br/>
        </xsl:if>
        <xsl:value-of select="Name"/>
        <br/>
        <xsl:call-template name="insertHtmlNewLines">
          <xsl:with-param name="value" select="Description"/>
        </xsl:call-template>
      </td>
    </tr>

    <tr class="only-mobile">
      <td colspan="3">
        <div>
          <xsl:apply-templates select="Join/Condition"/>
        </div>

        <xsl:if test="string-length(Title)>0">
          <xsl:value-of select="Title"/>
          <br/>
        </xsl:if>
        <xsl:value-of select="Name"/>
        <br/>
        <xsl:call-template name="insertHtmlNewLines">
          <xsl:with-param name="value" select="Description"/>
        </xsl:call-template>
      </td>
    </tr>
  </xsl:template>

  <!-- Relation to join condition -->
  <xsl:template match="RelationsTo/Relation/Join/Condition">
    <xsl:choose>
      <xsl:when test="IsSameObject='No'">
        <xsl:value-of select="OtherObjectName"/>
      </xsl:when>
      <xsl:otherwise>
        <span style="font-weight: bold">
          <xsl:value-of select="OtherObjectName"/>
        </span>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>.</xsl:text>
    <xsl:value-of select="OtherColumn"/>
    
    <xsl:text> = </xsl:text>
    
    <span style="font-weight: bold">
      <xsl:choose>
        <xsl:when test="$objectType = 'Table'">
          <xsl:value-of select="/DataDocumenter/Table/ObjectName"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="/DataDocumenter/View/ObjectName"/>
        </xsl:otherwise>
      </xsl:choose>
    </span>
    <xsl:text>.</xsl:text>
    <xsl:value-of select="CurrentColumn"/>
    <xsl:if test="position() != last()">
      <br/>
    </xsl:if>
  </xsl:template>

  <!-- Relation from -->
  <xsl:template match="RelationsFrom/Relation">
    <tr>
      <td>
        <xsl:choose>
          <xsl:when test="$objectType = 'Table'">
            <xsl:value-of select="/DataDocumenter/Table/ObjectName"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="/DataDocumenter/View/ObjectName"/>
          </xsl:otherwise>
        </xsl:choose>
      </td>

      <!-- relation icon -->
      <td>
        <xsl:choose>
          <xsl:when test="Type = 'USER'">
            <i class="icon-user-relation no-margin" title="User defined relation"></i>
          </xsl:when>
          <xsl:otherwise>
            <i class="icon-relation no-margin" title="Relation"></i>
          </xsl:otherwise>
        </xsl:choose>
      </td>

      <td>
        <xsl:call-template name="relationLink"/>
      </td>
      <td class="only-desktop">
        <xsl:apply-templates select="Join/Condition"/>
      </td>
      <td class="only-desktop">
        <xsl:if test="string-length(Title)>0">
          <xsl:value-of select="Title"/>
          <br/>
        </xsl:if>
        <xsl:value-of select="Name"/>
        <br/>
        <xsl:call-template name="insertHtmlNewLines">
          <xsl:with-param name="value" select="Description"/>
        </xsl:call-template>
      </td>
    </tr>

    <tr class="only-mobile">
      <td colspan="3">
        <div>
          <xsl:apply-templates select="Join/Condition"/>
        </div>

        <xsl:if test="string-length(Title)>0">
          <xsl:value-of select="Title"/>
          <br/>
        </xsl:if>
        <xsl:value-of select="Name"/>
        <br/>
        <xsl:call-template name="insertHtmlNewLines">
          <xsl:with-param name="value" select="Description"/>
        </xsl:call-template>
      </td>
    </tr>
  </xsl:template>

  <!-- Relation from join condition -->
  <xsl:template match="RelationsFrom/Relation/Join/Condition">
    <span style="font-weight: bold">
      <xsl:choose>
        <xsl:when test="$objectType = 'Table'">
          <xsl:value-of select="/DataDocumenter/Table/ObjectName"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="/DataDocumenter/View/ObjectName"/>
        </xsl:otherwise>
      </xsl:choose>
    </span>
    <xsl:text>.</xsl:text>
    <xsl:value-of select="CurrentColumn"/>
    
    <xsl:text> = </xsl:text>

    <xsl:choose>
      <xsl:when test="IsSameObject='No'">
        <xsl:value-of select="OtherObjectName"/>
      </xsl:when>
      <xsl:otherwise>
        <span style="font-weight: bold">
          <xsl:value-of select="OtherObjectName"/>
        </span>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>.</xsl:text>
    <xsl:value-of select="OtherColumn"/>
    
    <xsl:if test="position() != last()">
      <br/>
    </xsl:if>
  </xsl:template>

  <!-- Link for relation (for object other than current only) -->
  <xsl:template name="relationLink">
    <xsl:choose>
      <xsl:when test="IsExported = 'Yes' and
                (ObjectObjectId != /DataDocumenter/Table/ObjectId or
                  ObjectObjectId != /DataDocumenter/View/ObjectId)">

        <xsl:choose>
          <xsl:when test="IsInCurrentModule='Yes'">

            <xsl:variable name="moduleObjectId">
              <xsl:choose>
                <xsl:when test="$objectType = 'Table'">
                  <xsl:value-of select="/DataDocumenter/Table/ModuleObjectId"/>
                </xsl:when>
                <xsl:when test="$objectType = 'View'">
                  <xsl:value-of select="/DataDocumenter/View/ModuleObjectId"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="''"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>

            <xsl:choose>
              <xsl:when test="ObjectType = 'TABLE' and $moduleObjectId != ''">
                <a href="#modules/{$moduleObjectId}/tables/{ObjectObjectId}{$ext}" class="ajax" data-target="#right">
                  <xsl:value-of select="ObjectObjectName"/>
                </a>
              </xsl:when>
              <xsl:when test="ObjectType = 'VIEW' and $moduleObjectId != ''">
                <a href="#modules/{$moduleObjectId}/views/{ObjectObjectId}{$ext}" class="ajax" data-target="#right">
                  <xsl:value-of select="ObjectObjectName"/>
                </a>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="ObjectObjectName"/>
              </xsl:otherwise>
            </xsl:choose>

          </xsl:when>
          <xsl:otherwise>

            <xsl:choose>
              <xsl:when test="ObjectType = 'TABLE'">
                <a href="#tables/{ObjectObjectId}{$ext}" class="ajax" data-target="#right">
                  <xsl:value-of select="ObjectObjectName"/>
                </a>
              </xsl:when>
              <xsl:when test="ObjectType = 'VIEW'">
                <a href="#views/{ObjectObjectId}{$ext}" class="ajax" data-target="#right">
                  <xsl:value-of select="ObjectObjectName"/>
                </a>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="ObjectObjectName"/>
              </xsl:otherwise>
            </xsl:choose>

          </xsl:otherwise>
        </xsl:choose>

      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="ObjectObjectName"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- Unique keys -->
  <xsl:template match="UniqueKeys">
    <xsl:choose>
      <xsl:when test="./*">
        <table class="table">
          <caption>Unique keys</caption>
          <thead>
            <tr>
              <th style="width: 40px !important"></th>
              <th style="width: 20%">Key name</th>
              <th style="width: 20%">Columns</th>
              <th class="only-desktop">Description</th>
            </tr>
          </thead>
          <tbody>
            <xsl:apply-templates select="UniqueKey"/>
          </tbody>
        </table>
      </xsl:when>
    </xsl:choose>
  </xsl:template>

  <!-- Unique key -->
  <xsl:template match="UniqueKey">
    <tr>
      <xsl:choose>
        <xsl:when test="Type = 'PK'">
          <td>
            <i class="icon-key no-margin" title="Primary key"></i>
          </td>
        </xsl:when>
        <xsl:when test="Type = 'DisabledPK'">
          <td>
            <i class="icon-key no-margin" title="Disabled primary key"></i>
          </td>
        </xsl:when>
        <xsl:when test="Type = 'UserDefinedPK'">
          <td>
            <i class="icon-user-key no-margin" title="User defined primary key"></i>
          </td>
        </xsl:when>
        <xsl:when test="Type = 'UK'">
          <td>
            <i class="icon-unique-key no-margin" title="Unique key"></i>
          </td>
        </xsl:when>
        <xsl:when test="Type = 'DisabledUK'">
          <td>
            <i class="icon-unique-key no-margin" title="Disabled unique key"></i>
          </td>
        </xsl:when>
        <xsl:when test="Type = 'UserDefinedUK'">
          <td>
            <i class="icon-user-unique-key no-margin" title="User defined unique key"></i>
          </td>
        </xsl:when>
        <xsl:otherwise>
          <td>
          </td>
        </xsl:otherwise>
      </xsl:choose>
      <td>
        <xsl:value-of select="Name"/>
      </td>
      <td>
        <xsl:value-of select="Columns"/>
      </td>
      <td class="only-desktop">
        <xsl:call-template name="insertHtmlNewLines">
          <xsl:with-param name="value" select="Description"/>
        </xsl:call-template>
      </td>
    </tr>

    <xsl:if test="string-length(Description)>0">
      <tr class="only-mobile">
        <td colspan="3">
          <xsl:call-template name="insertHtmlNewLines">
            <xsl:with-param name="value" select="Description"/>
          </xsl:call-template>
        </td>
      </tr>
    </xsl:if>
  </xsl:template>

  <!-- Triggers -->
  <xsl:template match="Triggers">
    <xsl:choose>
      <xsl:when test="./*">
        <table class="table sortable">
          <caption>Triggers</caption>
          <thead>
            <tr>
              <th style="width: 40px !important"></th>
              <th style="width: 15%">Key name</th>
              <th style="width: 15%">When</th>
              <th class="only-desktop">Description</th>
            </tr>
          </thead>
          <tbody>
            <xsl:apply-templates select="Trigger"/>
          </tbody>
        </table>
      </xsl:when>
    </xsl:choose>
  </xsl:template>

  <!-- Trigger -->
  <xsl:template match="Trigger">
    <tr>
      <xsl:choose>
        <xsl:when test="Type = 'ActiveTrigger'">
          <td>
            <i class="icon-trigger-active no-margin" title="Active trigger"></i>
          </td>
        </xsl:when>
        <xsl:when test="Type = 'DisabledTrigger'">
          <td>
            <i class="icon-trigger-disabled no-margin" title="Disabled trigger"></i>
          </td>
        </xsl:when>
        <xsl:otherwise>
          <td>
          </td>
        </xsl:otherwise>
      </xsl:choose>
      <td>
        <xsl:value-of select="Name"/>
      </td>
      <td>
        <xsl:value-of select="When"/>
      </td>
      <td class="only-desktop">
        <xsl:call-template name="insertHtmlNewLines">
          <xsl:with-param name="value" select="Description"/>
        </xsl:call-template>
      </td>
    </tr>

    <xsl:if test="string-length(Description)>0">
      <tr class="only-mobile">
        <td colspan="3">
          <xsl:call-template name="insertHtmlNewLines">
            <xsl:with-param name="value" select="Description"/>
          </xsl:call-template>
        </td>
      </tr>
    </xsl:if>
  </xsl:template>

  <!-- Dependencies -->
  <xsl:template match="Dependencies">
    <xsl:apply-templates select="Uses">
      <xsl:with-param name="objectType" select="$objectType"/>
      <xsl:with-param name="content" select="$content"/>
      <xsl:with-param name="ext" select="$ext"/>
    </xsl:apply-templates>
    <xsl:apply-templates select="UsedBy">
      <xsl:with-param name="objectType" select="$objectType"/>
      <xsl:with-param name="content" select="$content"/>
      <xsl:with-param name="ext" select="$ext"/>
    </xsl:apply-templates>
  </xsl:template>
</xsl:stylesheet>